#include <string.h>
#include <stdio.h>


//#define ISBLACK(x) (x < 128)
#define ISBLACK(x) (x == BodyType)

#ifndef min
#define min(a,b) (((a) < (b)) ? (a) : (b))
#endif

extern char Message[];
extern void (*pMessageFunction)(char* AMessage, int Langage);
#define MESSAGE4(A,B,C,D) {sprintf(Message,A,B,C,D); pMessageFunction(Message, 0);}


// Helper function: counts the number of non-white pixels in "vertical" lines, with an inclination
// determined by inc (0 for vertical, .1f for 1:.1f slope).
void CountVert(int *VerCount, unsigned char **Line, int *Tops, int *Bots, int Length, int BodyType, 
			   float inc, int k)
{
	int i,j;
	float u;
	for(j=0;j<Length;j++)
	{
		VerCount[j] = 0;
		for(i=Bots[k],u=(float)j;i>=Tops[k];i--,u+=inc)
		{
			int IntU = (int)(u+0.1f);
			VerCount[j] += ISBLACK(Line[i][IntU]);
		}
	}
}

// Given the tally of non white pixels in vertical lines VerCount, counts how many characters are
// found in the line.
int CountChars(int* VerCount, int Length, int Height, float& AvSpace)
{
	int Lefts[128];
	int Rights[128];
	int NumChars;
	int i=0,j=0 ;
	while(j<Length)	
	{
		// Look for first non zero
		while(j<Length && VerCount[j] == 0) j++;
		if(j==Length) break;
		Lefts[i] = j++;
		
		// Now look for next zero one.
		while(j<Length && VerCount[j] != 0) j++;
		if(j==Length) break;
		Rights[i++] = j++;
	}
	NumChars = i;

	AvSpace = 0;
	for(i=0;i<NumChars-1;i++)
	{
		AvSpace += Lefts[i+1]-Rights[i];
	}
	AvSpace /= (NumChars-1);

	return NumChars;
}

// It's much better to count how many holes (white pix) we have in our vertical count, rather than how
// many characters we can count. This gives you a much more precise estimate of the angle for italics.
int CountHoles(int* VerCount, int Length, int Height)
{
	int count=0,j=0 ;
	for(j=0;j<Length;j++)
		if(VerCount[j] == 0) count++;
	return count;
}

// This straightens italicized characters (entire lines in fact). Inc is the slope.
void Fix(unsigned char **Line, int *Tops, int *Bots, int Length, float inc, int k)
{
	int i;
	float u;
	// Tops ans Bots are the top and bottom y for all lines.
	for(i=Tops[k],u=0;i<=Bots[k];i++,u+=inc)
	{
		int IntU = (int)(u+0.0f);
		// I'm moving the bottom lines to the right, rather than the top lines to the left, as
		// the latter tends to crop important parts of the leftmost character.
		memmove(Line[i]+IntU,Line[i],Length-IntU);
		memset(Line[i],0,IntU);
	}
}

void RemoveItalics(unsigned char* Bitmap,int Length, int Height, unsigned char BodyType, float& AverageSpace)
{
	int i,j,k;
	unsigned char *Line[1024];

	// Find horizontal lines. 
	int HorCount[1024];
	for(i=0;i<Height;i++)
	{
		HorCount[i] = 0;
		Line[i] = Bitmap+i*Length;
		for(j=0;j<Length;j++)
			HorCount[i] += ISBLACK(Line[i][j]);
	}
	// Run a little non-linear filter to fill in gaps, in case you have a line like "iiiii".
	for(i=2;i<Height-2;i++)
	{
		// Bridge gaps
		if(HorCount[i] == 0 && ((HorCount[i-1] || HorCount[i-2]) &&
			(HorCount[i+1] || HorCount[i+2])))
			HorCount[i] = 1;
	}
	// I now have the pixel count in each line. Find actual boundaries.
	int Tops[16];
	int Bots[16];
	i=j=0;
	while(i<Height)	
	{
		// Look for first non zero
		while(i<Height && HorCount[i] == 0) i++;
		if(i==Height) break;
		Tops[j] = i++;
		// Now look for next zero one.
		while(i<Height && HorCount[i] != 0) i++;
		if(i==Height) 
		{
			Bots[j++] = i-1;
			break;
		}
		Bots[j++] = i++;
	}
	int NumLines = j;
	
	int VerCount[1024];
	// For each line, detect if it's italicized and if so, straighten it.
	for(k=0;k<NumLines;k++)
	{
		// Count space between characters for verticals at various angles
		float Percents[4]={0,0.12f,0.22f,0.39f};
		int Italics = 0;
		float Per = 0;
		for(i=0;i<4;i++)
		{
			// The winner is the angle that yield the most blank space.
			CountVert(VerCount, Line, Tops, Bots, Length, BodyType,Percents[i],k);
			if(i==0 && k == 0)
				CountChars(VerCount,Length,Height,AverageSpace);
			int It = CountHoles(VerCount,Length,Height);
			if(It > Italics) Italics = It, Per = Percents[i];
		}
		if(Per > 0)
		{
//			MESSAGE4("Italics, %d vs %d - %.2f",Normal,Italics, Per);
			Fix(Line, Tops, Bots, Length, min(Per,0.33f), k);
		}
	}
}
