#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
//#include <unistd.h>

#include "pnm.h"
#ifdef HAVE_PAM_H
# include <pam.h>
# include <sys/types.h>
# include <sys/stat.h>
# include <fcntl.h>
#else
# include <ctype.h>
#endif

#include "gocr.h"
job_t *JOB;

void readpgm_mem(unsigned char* buffer,pix *p,int nx, int ny)
{	// raw-ppm
	p->p=buffer;  
	p->x=nx;  
	p->y=ny; 
	p->bpp=1;
}

static void out_version(int v) 
{
}


static void mark_start(job_t *job) {
  assert(job);

  if (job->cfg.verbose) {
    fprintf(stderr, "#");
    out_version(0);
    fprintf(stderr,
	    "# options are: -l %d -s %d -v %d -c %s -m %d -d %d -n %d %s\n",
	    job->cfg.cs, job->cfg.spc, job->cfg.verbose, job->cfg.lc, job->cfg.mode, 
            job->cfg.dust_size, job->cfg.only_numbers, job->src.fname);
#ifdef USE_UNICODE
    fprintf(stderr,"# using unicode\n");
#endif
#ifdef HAVE_GETTIMEOFDAY
    gettimeofday(&job->tmp.init_time, NULL);
#endif
  }
}

static void mark_end(job_t *job) {
  assert(job);

#ifdef HAVE_GETTIMEOFDAY
  /* show elapsed time */
  if (job->cfg.verbose) {
    struct timeval end, result;
    gettimeofday(&end, NULL);
    timeval_subtract(&result, &end, &job->tmp.init_time);
    fprintf(stderr,"Elapsed time: %d:%02d:%3.3f.\n", (int)result.tv_sec/60,
	(int)result.tv_sec%60, (float)result.tv_usec/1000);
  }
#endif
}

const char *getTextLine (int line);
void free_textlines(void);
int pgm2asc(job_t *job);


// subject of change, we need more output for XML (ToDo)
void print_output2(job_t *job, char (*Line)[128], int *NumLines) {
  int linecounter = 0;
  const char *line;

  assert(job);
        
  linecounter = 0;
  line = getTextLine(linecounter++);
  while (line) {
    // notice: decode() is shiftet to getTextLine since 0.38
    fputs(line, stdout);
	strcpy(Line[linecounter-1],line);
    line = getTextLine(linecounter++);
  }
  *NumLines = linecounter-1;
  free_textlines();
}


void RunOCR(unsigned char* Bitmap,int nx, int ny, char (*Line)[128], int *NumLines, int spc)
{
	job_t job;
	int i;

	JOB = &job;

	job_init(&job);

	mark_start(&job);
	job.cfg.spc = spc;
	// Enable this line to print the characters in /temp/ocrout.txt
//	job.cfg.verbose = 8;

	readpgm_mem(Bitmap,&job.src.p,nx, ny);

	pgm2asc(&job);

	print_output2(&job, Line, NumLines);

	mark_end(&job);

	free( job.tmp.ppo.p );

	/* FIXME jb: use box_free() instead of free() */
	list_and_data_free(&(job.res.boxlist), free);

	for(i=0;i<*NumLines;i++)
	{
		char* Ptr;
		while((Ptr = strstr(Line[i],"7.")) != 0)
		{
			*Ptr++ = '?';
			*Ptr = ' ';
		}
		while((Ptr = strstr(Line[i],"!.")) != 0)
		{
			*Ptr++ = '!';
			*Ptr = ' ';
		}
		while((Ptr = strstr(Line[i],"?.")) != 0)
		{
			*Ptr++ = '?';
			*Ptr = ' ';
		}
	}

}
